/*global globalTarget*/
/*global AJS*/

var SalesforceAuthenticationController = window.zkbangular.module("src", []);
SalesforceAuthenticationController.$inject = ["dataService","$scope", "$compile"];
SalesforceAuthenticationController.controller("SalesforceAuthenticationController", function (dataService,$scope, $compile) {
    var zd = this;
    zd.init = function () {
        zd.userCode = "";
        zd.verificationUri = "";
        zd.state = null;
        zd.loading = true;
        zd.loadingAuthorization = false;
        zd.showAuthHint = false;
        zd.showMessage = false;
        zd.showOauthOptions = false;
        zd.warning = false;
        zd.warningMessage = "";
        zd.errorMessage = "";
        zd.error = false;
        zd.showCodes = false;
    };

    zd.startDeviceAuthenticationFlow = function (state) {
        zd.showMessage = false;
        zd.state = state;
        zd.loading = true;
        zd.error = false;
        zd.showCodes = false;
        zd.disableSubmitButton();
        zd.showAuthorizationDialog();
        dataService.getVerificationToken(globalTarget, state).then(function (result) {
            if(result.warningMessage){
                zd.warningMessage = result.warningMessage;
                zd.warning = true;
                zd.loading=false;
                zd.showMessage = true;
            } else {
                zd.userCode = result.userCode;
                zd.verificationUri = result.verificationUri;
                zd.loading=false;
                zd.enableSubmitButton();
                zd.showCodes = true;
            }
        }).catch(function(response) {
            zd.loading = false;
            zd.error = true;
            zd.warning = false;
            zd.warningMessage = "";
            zd.showMessage = true;
            if(response.data && response.data.message){
                zd.errorMessage = response.data.message;
            } else {
                zd.errorMessage = "Unknown error";
            }

        }).finally(function() {});
    };

    zd.disableSubmitButton = function () {
        AJS.$("#dialog-submit-button").prop("disabled", true);
        AJS.$("#dialog-submit-button").prop("aria-disabled", "true");
    };

    zd.enableSubmitButton = function () {
        AJS.$("#dialog-submit-button").prop("disabled", false);
        AJS.$("#dialog-submit-button").prop("aria-disabled", false);
    };

    zd.authenticateAndAuthorize = function () {
        zd.disableSubmitButton();
        zd.loading = true;
        zd.showAuthHint = true;
        zd.showMessage = false;
        dataService.authenticateAndAuthorize(globalTarget, zd.userCode, zd.state).then(function (result) {
            zd.error = false;
            zd.warning = false;
            zd.loading = false;
            zd.showAuthHint = false;
            zd.errorMessage = "";
            zd.warningMessage = "";
            if(result.warningMessage){
                zd.warningMessage = result.warningMessage;
                zd.warning = true;
            }
        }).catch(function(response) {
            window.console.log("Authorization error", response);
            zd.loading = false;
            zd.error = true;
            zd.warning = false;
            zd.warningMessage = "";
            zd.showAuthHint = false;
            if(response.data && response.data.message){
                zd.errorMessage = response.data.message;
            } else {
                zd.errorMessage = "Unknown error";
            }

        }).finally(function() {
            zd.showMessage = true;
        });
    };

    zd.showAuthorizationDialog = function () {
        AJS.dialog2("#user-code-dialog").show();
    };

    zd.hideAuthorizationDialog = function () {
        AJS.dialog2("#user-code-dialog").hide();
        location.reload();
    };

    zd.logout = function (isPreview) {
        jQuery(isPreview? "#logoutPreviewContent": "#logoutContent").css({
            "display": "none"
        });

        dataService.logout(globalTarget, isPreview).then(function (result) {
           location.reload();
        }).catch(function (result){
            alert(result.data.errorMessage);
            location.reload();
        });
    };

    zd.openUriOnNewTabWin = function (event) {
        event.preventDefault();
        window.open(zd.verificationUri, "_blank");
    };
    zd.init();
});

function dataService($q,$http) {
    return {
        getVerificationToken (target, state){
            var deferred = $q.defer();
            var url =  AJS.contextPath() + "/rest/kb-internal/1.0/auth/get-device-token/" + target;
            if(typeof state !== "undefined" && state !== null){
                url += "?state=" + state;
            }
            var req = {
                method : "GET",
                url,
                headers : {
                    "Content-Type": "application/json"
                }
            };
            $http(req).then(function success(response) {
                deferred.resolve(response.data);
            }, function error(data) {
                deferred.reject(data);
            });
            return deferred.promise;
        },
        authenticateAndAuthorize (target, userCode, state) {
            var deferred = $q.defer();
            var url = AJS.contextPath() + "/rest/kb-internal/1.0/auth/authenticate-device/" + target + "?user_code=" + userCode;
            if(typeof state !== "undefined" && state !== null){
                url += "&state="+state;
            }
            var req = {
                method : "POST",
                url,
                headers : {
                    "Content-Type": "application/json"
                },
                data:{}
            };
            $http(req).then(function success(response) {
                deferred.resolve(response.data);
            }, function error(data) {
                deferred.reject(data);
            });
            return deferred.promise;
        },
        logout (target, isPreview) {
            const deferred = $q.defer();
            let url = AJS.contextPath() + "/rest/kb-internal/1.0/auth/logout/" + target + "?isPreview=" + isPreview;
            const req = {
                method : "POST",
                url,
                headers : {
                    "Content-Type": "application/json"
                },
                data:{}
            };
            $http(req).then(function success(response) {
                deferred.resolve(response.data);
            }, function error(data) {
                deferred.reject(data);
            });
            return deferred.promise;
        }
    };
}

SalesforceAuthenticationController.factory("dataService",dataService);
dataService.$inject = ["$q","$http"];
