var salesforceUpdateMetadata;

AJS.toInit(function() {
    AJS.$('#salesforceUpdateMetadata').click(function(e){
        e.preventDefault();

        if (AJS.$("#sfUpdateMetadata").length === 0) {
            initSalesforceUpdateCategoryOrChannelDialog();
        } else {
            salesforceUpdateMetadataController.internalInit();
        }
    });
});

var salesforceUpdateMetadataController;
var SalesforceUpdateMetadata = function($scope, sfUpdateMetadataService, KCSService, KFService){
    var sf = this;
    salesforceUpdateMetadataController = sf;
    sf.internalInit = function() {
        sf.maxEntries = 5;
        sf.hint = '';
        sf.contextPath = AJS.contextPath();
        sf.isShowing = {
            mainForm: false,
            wait: false,
            loader: true
        };
        sf.error = false;
        sf.errorMessage = "";
        sf.warning = false;
        sf.warningDetails = "";
        sf.warningMessage = "";
        sf.success = false;
        sf.successMessage = "";
        sf.updateButtonDisabled = true;
        sf.updateButtonLabel = 'Save as Draft';
        sf.publishChangesButtonDisabled = true;
        sf.publishChangesButtonLabel = 'Publish Changes';
        sf.data = {};
        sf.title = "Update Salesforce Metadata";
        sf.waitIconUrl = sf.contextPath + '/images/icons/wait.gif';

        AJS.dialog2("#sfUpdateMetadata").show();

        var allCategoriesData;
        var articleData;

        sfUpdateMetadataService.categories().then(function (result){
            allCategoriesData = result;
           return sfUpdateMetadataService.metadata();
        }).catch(function (reason) {
            sf.error = true;
            sf.errorMessage = 'Error retrieving all categories. Please close the Pop-up and try again';
            console.log('Error retrieving all categories ', reason);
            return false;
        }).then(function (result){
            if(!result){
                return false;
            }

            articleData = result;

            sf.data = {
                categories: allCategoriesData
            };

            KCSService.setCategoriesMetadata(sf.data);

            KFService.setMetadata({
                recordTypes:
                    articleData['record-types'].records.map(function(recordType){
                    return {
                        id: recordType.Id,
                        name: recordType.Name
                    }
                }),
                articleType: articleData.article.attributes.type.replace('__kav', '')
            });

            var channels = [];
            if(articleData.article.IsVisibleInPkb){
                channels.push("IsVisibleInPkb");
            }
            if(articleData.article.IsVisibleInCsp){
                channels.push("IsVisibleInCsp");
            }
            if(articleData.article.IsVisibleInPrm){
                channels.push("IsVisibleInPrm");
            }

            KFService.setData({
                channels: channels,
                recordType: {
                    id: articleData.article.RecordTypeId
                }
            });

            var categories = articleData.categories.records.map(
                function (category) {
                    return {
                        category: category.DataCategoryName,
                        categoryGroup: category.DataCategoryGroupName
                    };
                }
            );

            KCSService.setSelectedCategories( categories );

            sf.isShowing.mainForm = true;
            sf.updateButtonDisabled = false;


            channels.push("IsVisibleInApp");
            sf.originalValues = {
                channels: channels,
                recordType: {
                    id: articleData.article.RecordTypeId
                },
                categories: categories
            };

        }).catch(function (reason) {
            sf.success = false;
            sf.successMessage = "";
            sf.error = true;
            sf.errorMessage = 'Error retrieving Article Metadata. Please close the Pop-up and try again';
            console.log('Error retrieving Article Metadata ', reason);
        }).finally( function () {
            sf.isShowing.loader = false;
        });
    };
    sf.closeDialog = function(){
        sfUpdateMetadataService.cancel();
        AJS.dialog2("#sfUpdateMetadata").hide();
        if(sf.success){
            window.location.reload();
        }
    };
    sf.detectChanges = function(newValues){
        if(newValues.recordType && newValues.recordType.id!=sf.originalValues.recordType.id){
            return true;
        }
        if(newValues.channels.length != sf.originalValues.channels.length){
            return true;
        }else{
            for(var j = 0;j<newValues.channels.length ;j++){
                var newChannel = newValues.channels[j];
                if(sf.originalValues.channels.indexOf(newChannel)==-1){
                    return true;
                }
            }

        }
        if(newValues.categories.length != sf.originalValues.categories.length){
            return true;
        }else{
            var compare = function(category){
                return category.categoryGroup == newCategory.categoryGroup &&
                    category.category == newCategory.category;
            }
            for(var j = 0;j<newValues.categories.length ;j++){
                var newCategory = newValues.categories[j];
                if(!sf.originalValues.categories.find(compare)){
                    return true;
                }
            }
        }

        return false;
    }
    sf.updateMetadata = function(){
        var request = {
            channels: KFService.selectedChannels(),
            recordType: KFService.selectedRecordType(),
            categories: KCSService.getSelectedCategories()
        };

        sf.hint="";
        if(!sf.detectChanges(request)){
            sf.hintStyle={"color":"red"};
            sf.hint = "There are no changes to be made!";
            return;
        }

        sf.updateButtonDisabled = true;
        sf.isShowing.wait = true;

        sfUpdateMetadataService.updateArticleMetadata(request)
            .then(function(result){
                console.log("Update Result ", result);

                if( result["warning"] ){
                    sf.warning = true;
                    sf.warningMessage = '';
                    sf.warningDetails = JSON.stringify(result["warning"]);
                    return;
                }

                if( result["updateCategoriesResult"] && result["updateCategoriesResult"]["add-response"] ){
                    var addResponse = result["updateCategoriesResult"]["add-response"];
                    var i = 0;
                    for(; i < addResponse.length; i++ ){
                        if(!addResponse[i].success){
                            sf.warning = true;
                            sf.warningMessage = 'Incomplete update. Please close and open the Pop-up ' +
                                'to complete the update';
                            sf.warningDetails = JSON.stringify(addResponse[i].errors[0]);
                            return;
                        }
                    }
                }

                sf.success = true;
                sf.successMessage = 'Article Metadata saved as Draft';

                sf.publishChangesButtonDisabled = false;
            })
            .catch(function(reason){
                console.error("Update Error", reason);
                sf.error = true;
                sf.errorMessage = 'Error saving Article Metadata. Please close the Pop-up and try again';
            })
            .finally(function(){
                sf.isShowing.wait = false;
            });
    };
    sf.publishChanges = function(){
        sf.publishChangesButtonDisabled = true;
        sf.isShowing.wait = true;

        sfUpdateMetadataService.publishChanges()
            .then(function(result){
                console.log("Public Changes Result ", result);
                sf.success = true;
                sf.successMessage = 'Changes published! You can now close this Pop-up';
            })
            .catch(function(reason){
                console.error("Public Changes Error", reason);
                sf.error = true;
                sf.errorMessage = 'Error publishing changes. Please close the Pop-up and try again';
            })
            .finally(function(){
                sf.isShowing.wait = false;
            });
    };

    sf.internalInit();
};

var sfUpdateMetadataService = function($q, $http) {
    return {
        promises: {
            categories: null,
            metadata: null,
            updateArticleMetadata: null,
            publishChanges: null
        },
        cancel: function(){
            if(this.promises.categories){
                this.promises.categories.resolve();
            }
            if(this.promises.metadata){
                this.promises.metadata.resolve();
            }
            if(this.promises.updateArticleMetadata){
                this.promises.updateArticleMetadata.resolve();
            }
            if(this.promises.publishChanges){
                this.promises.publishChanges.resolve();
            }
        },
        categories: function(){
            var deferred = $q.defer();
            var req = {
                method: "GET",
                url: AJS.contextPath() + "/rest/kb-internal/1.0/sfdc-utils/metadata/all-categories",
                headers: {
                    'Content-Type': "application/json"
                }
            };
            $http(req).then(function success(response) {
                deferred.resolve(response.data);
            }, function error(data) {
                deferred.reject(data);
            });
            this.promises.categories = deferred;
            return deferred.promise;
        },
        metadata: function() {
            var deferred = $q.defer();
            var req = {
                method: "GET",
                url: AJS.contextPath() + "/rest/kb-internal/1.0/sfdc-utils/metadata/" + AJS.params.pageId,
                headers: {
                    'Content-Type': "application/json"
                }
            };
            $http(req).then(function success(response) {
                deferred.resolve(response.data);
            }, function error(data) {
                deferred.reject(data);
            });
            this.promises.metadata = deferred;
            return deferred.promise;
        },
        updateArticleMetadata: function(request) {
            var deferred = $q.defer();
            var req = {
                method: "POST",
                url: AJS.contextPath() + "/rest/kb-internal/1.0/sfdc-utils/metadata/" + AJS.params.pageId,
                headers: {
                    'Content-Type': "application/json"
                },
                data: request
            };
            $http(req).then(function success(response) {
                deferred.resolve(response.data);
            }, function error(data) {
                deferred.reject(data);
            });
            this.promises.updateArticleMetadata = deferred;
            return deferred.promise;
        },
        publishChanges: function() {
            var deferred = $q.defer();
            var req = {
                method: "POST",
                url: AJS.contextPath() + "/rest/kb-internal/1.0/sfdc-utils/publish/" + AJS.params.pageId,
                headers: {
                    'Content-Type': "application/json"
                },
                data: {}
            };
            $http(req).then(function success(response) {
                deferred.resolve(response.data);
            }, function error(data) {
                deferred.reject(data);
            });
            this.promises.publishChanges = deferred;
            return deferred.promise;
        }
    }
};


function initSalesforceUpdateCategoryOrChannelDialog() {
    //--inject dialog with angular component to body
    var dialog2Str = '';
    dialog2Str += '<section role="dialog" id="sfUpdateMetadata" class="aui-layer aui-dialog2 aui-dialog2-medium" aria-hidden="true" data-aui-modal="true" style="width: 650px;">';
    dialog2Str += '<div ng-app="sf-update-metadata">';
    dialog2Str += '<salesforce-update-metadata>';
    dialog2Str += '</salesforce-update-metadata>';
    dialog2Str += '</div>';
    dialog2Str += '</section>';
    AJS.$(document.body).append(AJS.$(dialog2Str));

    //--create angular app, initiate component and relate controller and services
    salesforceUpdateMetadata = window.zkbangular.module("sf-update-metadata", []);
    salesforceUpdateMetadata.factory("sfUpdateMetadataService", sfUpdateMetadataService);
    sfUpdateMetadataService.$inject = ["$q", "$http"];

    loadKnowledgeCategoryComponent(salesforceUpdateMetadata);
    loadKnowledgeFieldsComponent(salesforceUpdateMetadata);

    salesforceUpdateMetadata.component("salesforceUpdateMetadata", {
        templateUrl: AJS.Data.get("context-path") +
            '/download/resources/com.zagile.confluence.kb.knowledge-bridge:salesforce-modal-resources/salesforce-update-metadata.html',
        controller: ['$scope', 'sfUpdateMetadataService', 'KCSService', 'KFService', SalesforceUpdateMetadata]
    });

    //--compile angular app
    var injector = window.zkbangular.injector(["ng", "sf-update-metadata"]);
    injector.invoke(function($compile, $rootScope) {
        $compile(AJS.$("div[ng-app='sf-update-metadata']"))($rootScope);
        $rootScope.$apply();
    });

}
