let zAJS;
if (typeof window.AJS !== "undefined") {
    zAJS = window.AJS;
}
let zAngular;
if (typeof window.zkbangular !== "undefined") {
    zAngular = window.zkbangular;
}
const DAYS_REMAINING_TO_WARN = 30;
const PAGE_SIZE = 10;
const ANIMATION_DURATION = 5000;

let ZUnpublishController = zAngular.module("src", []);
ZUnpublishController.$inject = ["dataService"];

ZUnpublishController.controller("ZUnpublishController", function (dataService) {
    let zkb = this;
    zkb.init = () => {
        zkb.isLicensed = false;
        zkb.stopMonitoring = false;
        zkb.showGeneralError = false;
        zkb.generalErrorMessage = "";
        zkb.contentResponse = {};
        zkb.contentResults = [];
        zkb.page = 1;
        zkb.navigateToPage = 1;
        zkb.totalPages = 0;
        zkb.showLoader = true;
        zkb.selection = {};
        zkb.selectionIds = [];
        zkb.summary = null;
        zkb.spaceKey = zAJS.params.spaceKey;

        zkb.validateLicense().then(zkb.bulkUnpublishInfo).catch(zkb.writeGeneralError).finally(() => {
            zkb.showLoader = false;
        });
    };
    zkb.bulkUnpublishInfo = () => {
        let startAt = PAGE_SIZE * (zkb.navigateToPage - 1);
        return dataService.bulkUnpublishInfo(startAt).then((result) => {
            zkb.summary = result;
            zkb.totalPages = 0;
            if (!result.inProgress) {
                if (zkb.stopMonitoring) {
                    return true;
                }
                zkb.page = zkb.navigateToPage;
                zkb.contentResponse = result.response;
                zkb.contentResults = zkb.contentResponse.results;
                zkb.totalPages = Math.ceil(zkb.contentResponse.totalSize / PAGE_SIZE) | 0;
            } else if (result.inProgress && zkb.summary.space.key === zkb.spaceKey) {
                zkb.stopMonitoring = true;

                let progress = zkb.summary.totalToProcess === 0 ? 0 :
                    zkb.summary.totalProcessed / zkb.summary.totalToProcess;

                if (progress > 0) {
                    zAJS.progressBars.update("progress", progress);
                }

                setTimeout(() => {
                    zkb.bulkUnpublishInfo().catch(zkb.writeGeneralError);
                }, ANIMATION_DURATION);
            }
        });
    };
    zkb.writeGeneralError = (response) => {
        zkb.showGeneralError = true;

        if (response.data && response.data.message) {
            zkb.generalErrorMessage = response.data.message;
        } else if (response.statusText) {
            zkb.generalErrorMessage = response.statusText;
        } else {
            zkb.generalErrorMessage = "Unknown Error";
        }
    };
    zkb.navigate = (page) => {
        zkb.showLoader = true;
        zkb.navigateToPage = page;
        zkb.bulkUnpublishInfo().catch(zkb.writeGeneralError).finally(() => {
            zkb.showLoader = false;
        });
    };
    zkb.previousPage = () => {
        zkb.navigate(zkb.page - 1);
    };
    zkb.goToPage = (page) => {
        zkb.navigate(page);
    };
    zkb.nextPage = () => {
        zkb.navigate(zkb.page + 1);
    };
    zkb.validateLicense = () => {
        return dataService.isSfdcLicensed().then((isSfdcLicensed) => {
            if (isSfdcLicensed.daysToExpire && isSfdcLicensed.daysToExpire !== -1 &&
                isSfdcLicensed.daysToExpire <= DAYS_REMAINING_TO_WARN) {
                zAJS.flag({
                    type: "warning",
                    body: `Your Knowledge:Bridge license will expire in ${isSfdcLicensed.daysToExpire} days.`,
                });
            }
            zkb.isLicensed = true;
        });
    };
    zkb.selectAll = () => {
        if (zkb.contentResults) {
            for (let content of zkb.contentResults) {
                zkb.selection[content.id] = true;
            }
        }
    };
    zkb.clearPage = () => {
        if (zkb.contentResults) {
            for (let content of zkb.contentResults) {
                delete zkb.selection[content.id];
            }
        }
    };
    zkb.clearAll = () => {
        zkb.selection = {};
    };
    zkb.unpublish = () => {
        zkb.selectionIds = [];
        for (let contentId in zkb.selection) {
            if (Object.prototype.hasOwnProperty.call(zkb.selection, contentId)) {
                zkb.selectionIds.push(contentId);
            }
        }
        zkb.showGeneralError = false;
        if (zkb.selectionIds.length === 0) {
            zkb.generalErrorMessage = "Select at least one content to unpublish";
            zkb.showGeneralError = true;
            return;
        }
        zAJS.dialog2("#confirmDialog").show();
    };
    zkb.unpublishAll = () => {
        zkb.selectionIds = [];
        zAJS.dialog2("#confirmDialog").show();
    };
    zkb.cancel = () => {
        zAJS.dialog2("#confirmDialog").hide();
    };
    zkb.confirm = () => {
        zAJS.dialog2("#confirmDialog").hide();
        zkb.showLoader = true;
        zkb.navigateToPage = 1;
        zkb.summary = null;
        zkb.totalPages = 0;
        return dataService.unpublish(zkb.selectionIds).then(() => {
            zkb.bulkUnpublishInfo().catch(zkb.writeGeneralError);
        }).catch(zkb.writeGeneralError).finally(() => {
            zkb.showLoader = false;
        });
    };
    zkb.storeSelectionInfo = (content) => {
        if (!zkb.selection[content.id]) {
            delete zkb.selection[content.id];
        }
    };
    zkb.init();
});

const executeRequest = ($http, req, defer) => {
    $http(req).then(
        (success) => {
            defer.resolve(success.data);
        },
        (error) => {
            defer.reject(error);
        }
    );
    return defer.promise;
};

const dataService = ($q, $http) => {
    return {
        isSfdcLicensed: () => {
            return executeRequest($http, {
                method: "GET",
                url: `${zAJS.contextPath()}/rest/kb-internal/1.0/sfdc-utils/isSfdcLicensed`,
                headers: {
                    "Content-Type": "application/json"
                }
            }, $q.defer());

        },
        bulkUnpublishInfo: (start) => {
            return executeRequest($http, {
                method: "GET",
                url: `${zAJS.contextPath()}/rest/kb-internal/1.0/sfdc-utils/bulkUnpublishInfo/${zAJS.params.spaceKey}` +
                    `?start=${start}&limit=${PAGE_SIZE}`,
                headers: {
                    "Content-Type": "application/json"
                }
            }, $q.defer());
        },
        unpublish: (request) => {
            return executeRequest($http, {
                method: "POST",
                data: request,
                url: `${zAJS.contextPath()}/rest/kb-internal/1.0/sfdc-utils/unpublish/${zAJS.params.spaceKey}`,
                headers: {
                    "Content-Type": "application/json"
                }
            }, $q.defer());
        }
    };
};

ZUnpublishController.factory("dataService", dataService);
dataService.$inject = ["$q", "$http"];

zAJS.$(document).ready(() => {
    zAJS.$("div[ng-app='src']").removeClass("hidden");
});