var updateElement = function(element, disabled) {
	element.disabled = disabled;
	element.checked = false;
	if (window.zkbangular.isArray(element.categories)) {
		window.zkbangular.forEach(element.categories, function (childElement) {
			updateElement(childElement, disabled);
		});
	}
};

var setCategory = function(category) {
	if (!category.isGroup) {
		if (window.zkbangular.isArray(category.categories)) {
			window.zkbangular.forEach(category.categories, function (childCategory) {
				updateElement(childCategory, category.checked);
			});
		}
	}
};

var selectedCategories = function(group,category,selected) {
	if (category.checked) {
		selected.push({"categoryGroup": group, "category": category.name});
		return;
	}
	window.zkbangular.forEach(category.categories, function (child) {
		selectedCategories(group, child, selected);
	});
};

var KCSController = function($rootScope, $scope, $q, KCSService) {
	var sf = this;

	sf.errorMessage;

	sf.categories = [];

	sf.$onInit = function() {
        $rootScope['KCSController'] = sf;
	};

	sf.checkCategories=function(categories){
        var found;
		var markCategory=function(markCategory){
			found = false;
			var markCategoryInternal = function(category){
				if(found){
					return;
				}
				if(category.group == markCategory.categoryGroup && category.name == markCategory.category){
                    category.checked = true;
                    found = true;
                    return;
				}
				if(category.categories) {
					for (var i = 0; i < category.categories.length; i++) {
						var child = category.categories[i];
						markCategoryInternal(child);
						if (found) {
							sf.expandSingle(child);
							break;
						}
					}
				}
			};
			window.zkbangular.forEach(sf.categories, function (category) {
				markCategoryInternal(category);
			});
		};

		categories.forEach(function(category){
			markCategory(category);
		});
	};

	sf.salesforceCategories = function(result) {
		sf.categories = result["categories"];

		window.zkbangular.forEach(sf.categories, function (category) {
			category.isGroup = true;
			category.visible = true;
			category.expanded = true;
			category.checked = false;
			category.level = 0;
			category.isLeaf = false;

			window.zkbangular.forEach(category.categories, function (child) {
				sf.setFlagFields(category.level + 1, category.name, null, child);
			});
		});
	}

	sf.setFlagFields = function(level, group, parent, category) {
		category.group = group;
		category.parent = parent;
		category.isGroup = false;
		category.visible = (level == 1);
		category.expanded = false;
		category.checked = false;
		category.level = level;

		if (window.zkbangular.isArray(category.categories)) {
			category.isLeaf = false;
		} else {
			category.isLeaf = true;
		}

		window.zkbangular.forEach(category.categories, function (child) {
			sf.setFlagFields(category.level + 1, group, category.label, child);
		});
	}

	sf.collapseAll = function() {
		window.zkbangular.forEach(sf.categories, function (node) {
			sf.collapse(node);
		});
	}

	sf.expandAll = function() {
		window.zkbangular.forEach(sf.categories, function (node) {
			sf.expand(node);
		});
	}

    sf.expandSingle = function(category) {
		category.expanded = true;
		category.visible = true;
		window.zkbangular.forEach(category.categories, function (node) {
			node.visible = true;
		});
	}

	sf.expand = function(category) {
		category.expanded = true;
		category.visible = true;
		window.zkbangular.forEach(category.categories, function (node) {
			sf.expand(node);
		});
	}

	sf.collapse = function(category) {
		category.expanded = false;
		if (!category.isGroup) {
			category.visible = false;
		}
		window.zkbangular.forEach(category.categories, function (node) {
			sf.collapse(node);
		});
	}
};

var KCSService = function($rootScope, $q, $http) {
	return {
		
		getSelectedCategories: function() {

			//Very Ugly! :/
			//var categories = $rootScope["$$childHead"]["$$childHead"]["$ctrl"]["categories"];
			//End Very Ugly :/

			//Beautiful
			var categories = $rootScope['KCSController']["categories"];
			//End Beautiful

			var selected = [];
			window.zkbangular.forEach(categories, function (category) {
				selectedCategories(category.name, category, selected);
			});
			return selected;
		},		
		setCategoriesMetadata: function(metadata) {
			var sf =$rootScope['KCSController'];
			sf.salesforceCategories(metadata);
		},
        setCategories:function(categories){
            var sf =$rootScope['KCSController'];
            sf.checkCategories(categories);
        },
        setSelectedCategories: function(categories){
            var sf =$rootScope['KCSController'];
            sf.checkCategories(categories);
        }
	};
};

function loadKnowledgeCategoryComponent(app) {

	var zAppTree = app;

	zAppTree.factory("KCSService", KCSService);
	KCSService.$inject = ["$rootScope", "$q", "$http"];

	zAppTree.component("kcselector", {
		templateUrl: AJS.Data.get("context-path") +
			'/download/resources/com.zagile.confluence.kb.knowledge-bridge:knowledge-category-selector/knowledge-category-selector.html',
		controller: ['$rootScope', '$scope', '$q', 'KCSService', KCSController],
	});

	zAppTree.directive('htree', function() {
		return {
			restrict: "E",
			replace: true,
			scope: {
				collection: '='
			},
			template: "<ul style='margin-left: -60px; margin-top: -10px;'><tree collection='collection'></tree></ul>"
		}
	}).directive('tree', function() {
		return {
			restrict: "E",
			replace: true,
			scope: {
				collection: '='
			},
			template: "<ul style='list-style-type: none; margin: 3px; margin-left: 5px;'><node ng-repeat='element in collection' element='element'></node></ul>"
		}
	}).directive('node', function($compile) {
		return {
			restrict: "E",
			replace: true,
			scope: {
				element: '='
			},
			template: "<li class='node' ng-show='element.visible' style='white-space: nowrap;'><span ng-show='element.expanded && !element.isLeaf' ng-click='collapse(element)' class='kcs-icon kcs-icon-expanded'></span><span ng-show='!element.expanded && !element.isLeaf' ng-click='expand(element)' class='kcs-icon kcs-icon-collapsed'></span><span ng-show='element.isLeaf' class='kcs-icon kcs-icon-leaf'></span><input id='{{ element.name }}' type='checkbox' ng-show='!element.isGroup' ng-change='addCategory(element)' ng-model='element.checked' ng-disabled='element.disabled'/><label style='padding-left: 4px;' for='{{ element.name }}' ng-disabled='element.disabled'>{{ element.label }}</label></li>",
			link: function(scope, element, attrs) {

				scope.addCategory = function(member) {
					setCategory(member);
				};

				scope.expand = function(node) {
					node.expanded = true;
					scope.setVisible(node.categories, node.expanded);
				}

				scope.collapse = function(node) {
					node.expanded = false;
					scope.setVisible(node.categories, node.expanded);
				}

				scope.setVisible = function (nodes, visible) {
					if (window.zkbangular.isArray(nodes)) {
						window.zkbangular.forEach(nodes, function (node) {
							node.visible = visible;
						});
					}
				};

				if (window.zkbangular.isArray(scope.element.categories)) {
					element.append("<tree collection='element.categories'></tree>");
					$compile(element.contents())(scope)
				}
			}
		}
	}).directive('list', function($compile) {
		return {
			restrict: "E",
			replace: true,
			scope: {
				collection: '='
			},
			template: "<ul style='margin-left: -45px; margin-top: -13px;'><item ng-repeat='element in collection' element='element'></item></ul>"
		}
	}).directive('item', function($compile) {
		return {
			restrict: "E",
			replace: true,
			scope: {
				element: '='
			},
			template: "<li class='item' style='padding-bottom: 5px; padding-top: 5px; margin-left: 5px; border-bottom: 1px solid #cccccc;' ng-show='!element.isGroup && element.checked'><span class='aui-icon aui-icon-small aui-iconfont-cross-circle' style='margin-top: -2px; margin-left: 5px; margin-right: 5px; cursor: pointer;' ng-click='removeCategory(element)'></span><span style='color: #7d7d7d;'>{{element.group}} <span ng-show='element.parent != null'><span ng-show='element.level > 2'>...</span> → {{element.parent}}</span> → {{element.label}}</span></li>",
			link: function(scope, element, attrs) {

				scope.removeCategory = function (category) {
					category.checked = false;
					setCategory(category);
				};

				if (window.zkbangular.isArray(scope.element.categories)) {
					element.after($compile("<item ng-repeat='element in element.categories' element='element'></item>")(scope));
				}
			}
		}
	});
}