var knowledgeSalesforceFeedbackCommon = {
  promises: {
    salesforceFeedback: null,
    confluenceFeedback: null,
    editMeta: null,
    isChangedInSalesforce: null,
    metadata: null,
    categories: null
  },
  cancel: function(){
    if( this.promises.salesforceFeedback ){
      this.promises.salesforceFeedback.resolve();
    }
    if( this.promises.confluenceFeedback ){
      this.promises.confluenceFeedback.resolve();
    }
    if( this.promises.editMeta ){
      this.promises.editMeta.resolve();
    }
    if( this.promises.isChangedInSalesforce ){
      this.promises.isChangedInSalesforce.resolve();
    }
    if( this.promises.metadata ){
      this.promises.metadata.resolve();
    }
    if( this.promises.categories ){
      this.promises.categories.resolve();
    }
  },
  salesforceFeedback: function( $q, $http ) {
    var deferred = $q.defer();
    var req = {
      method: "GET",
      url: AJS.contextPath() + "/rest/kb-internal/1.0/sfdc-utils/salesforce-feedback/" + AJS.params.pageId,
      headers: {
        'Content-Type': "application/json"
      }
    };
    $http(req).then(function success(response) {
      deferred.resolve(response.data);
    }, function error(data) {
      deferred.reject(data);
    });
    this.promises.salesforceFeedback = deferred;
    return deferred.promise;
  },
  confluenceFeedback: function( $q, $http ) {
    var deferred = $q.defer();
    var req = {
      method: "GET",
      url: AJS.contextPath() + "/rest/kb-internal/1.0/confluence-utils/confluence-feedback/" + AJS.params.pageId,
      headers: {
        'Content-Type': "application/json"
      }
    };
    $http(req).then(function success(response) {
      deferred.resolve(response.data);
    }, function error(data) {
      deferred.reject(data);
    });
    this.promises.confluenceFeedback = deferred;
    return deferred.promise;
  },
  editMeta: function( $q, $http ) {
    var deferred = $q.defer();
    var req = {
      method: "GET",
      url: AJS.contextPath() + "/rest/kb-internal/1.0/content/" + AJS.params.pageId + "/salesforce/page-editmeta",
      headers: {
        'Content-Type': "application/json"
      }
    };
    $http(req).then(function success(response) {
      deferred.resolve(response.data);
    }, function error(data) {
      deferred.reject(data);
    });
    this.promises.editMeta = deferred;
    return deferred.promise;
  },
  isChangedInSalesforce:function( $q, $http ){
    var deferred = $q.defer();
    var req = {
      method: "GET",
      url: AJS.contextPath()+"/rest/kb-internal/1.0/sfdc-utils/detect/"+AJS.params.pageId,
      headers: {
        'Content-Type': "application/json"
      }
    };
    $http(req).then(function success(response) {
      deferred.resolve(response.data);
    }, function error(data) {
      deferred.reject(data);
    });
    return deferred.promise;
  },
  metadata: function( $q, $http ) {
    var deferred = $q.defer();
    var req = {
      method: "GET",
      url: AJS.contextPath() + "/rest/kb-internal/1.0/sfdc-utils/metadata/" + AJS.params.pageId,
      headers: {
        'Content-Type': "application/json"
      }
    };
    $http(req).then(function success(response) {
      deferred.resolve(response.data);
    }, function error(data) {
      deferred.reject(data);
    });
    this.promises.metadata = deferred;
    return deferred.promise;
  },
  categories: function( $q, $http ){
    var deferred = $q.defer();
    var req = {
      method: "GET",
      url: AJS.contextPath() + "/rest/kb-internal/1.0/sfdc-utils/metadata/all-categories",
      headers: {
        'Content-Type': "application/json"
      }
    };
    $http(req).then(function success(response) {
      deferred.resolve(response.data);
    }, function error(data) {
      deferred.reject(data);
    });
    this.promises.categories = deferred;
    return deferred.promise;
  },
  calculateKAVsToShow: function( $scope,
                                 knowledgeArticleVersionId,
                                 draftKnowledgeArticleVersionId,
                                 onlineKnowledgeArticleVersionId){
  var kav = knowledgeArticleVersionId;
    var draftKav = draftKnowledgeArticleVersionId;
    var onlineKav = onlineKnowledgeArticleVersionId;

    $scope.resultPageShowStoredKAV = kav && true;
    if(!kav || ( kav !== draftKav && kav !== onlineKav )){
      $scope.resultPageShowDrafKAV = draftKav && true;
      $scope.resultPageShowOnlineKAV = onlineKav && true;
    }
  },
  updateScopeWithArticleMetadata: function($scope, data ){

    $scope.articleNumber = data.article.ArticleNumber;
    $scope.publishStatus = data.article.PublishStatus;
    if( data.article.VersionNumber === 0 ){
      $scope.versionNumber = "Draft Version";
    }
    else{
      $scope.versionNumber = data.article.VersionNumber;
    }

    if(!data.article.attributes){
      return;
    }

    $scope.articleType = data.article.attributes.type.replace('__kav', '');
    if(data.article.RecordTypeId){
      for(var i = 0; i < data['record-types'].records.length; i++){
        if( data['record-types'].records[i].Id === data.article.RecordTypeId ){
          $scope.recordType = data['record-types'].records[i].Name;
          break;
        }
      }
    }

    var channelsNames = [];
    if( data.article.IsVisibleInApp ){
      channelsNames.push("Internal App");
    }
    if( data.article.IsVisibleInPrm){
      channelsNames.push("Partner");
    }
    if( data.article.IsVisibleInCsp ){
      channelsNames.push("Customer");
    }
    if( data.article.IsVisibleInPkb ){
      channelsNames.push("Public Knowledge Base");
    }
    $scope.channelsNames = channelsNames;

  }
};