var knowledgeSalesforceFeedbackDetailsController =
    function(
        $rootScope,
        $scope,
        $element,
        $q,
        $timeout,
        knowledgeSalesforceFeedbackDetailsServiceInstance ){

        $scope.$parent.detailsController = this;
        $scope.contextPath = AJS.contextPath();

        $scope.isAnyRootExpanded = function(){
            if( !$scope.rootCategories ){
                return false;
            }
            for(var i = 0; i < $scope.rootCategories.length; i++){
                if($scope.rootCategories[i].expanded){
                    return true;
                }
            }
            return false;
        };

        $scope.handleExpandAll = function(){
            if( !$scope.rootCategories ){
                return;
            }
            for(var i = 0; i < $scope.rootCategories.length; i++){
                $scope.rootCategories[i].expanded = true;
            }
        };
        $scope.handleCollapseAll = function(){
            if( !$scope.rootCategories ){
                return;
            }
            for(var i = 0; i < $scope.rootCategories.length; i++){
                $scope.rootCategories[i].expanded = false;
            }
        };

        var buildPathFragments = function (root, name, pathFragments, labelsInPathIndex) {
            if (root.name === name) {
                if (root.label !== 'All') {
                    pathFragments.push(root.label);
                    labelsInPathIndex.set(root.name,root.label);
                }
                return true;
            }

            if (root.topCategories) {
                root.childCategories = root.topCategories;
            }

            if (root.childCategories) {
                for (let childCategory of root.childCategories) {
                    if (buildPathFragments(childCategory, name, pathFragments, labelsInPathIndex)) {
                        if (root.label !== 'All') {
                            pathFragments.push(root.label);
                            labelsInPathIndex.set(root.name,root.label);
                        }
                        return true;
                    }
                }
            }
            return false;
        };


        var handleCategories = function () {
            let rootCategories = [];
            let rootCategoryIndex = new Map();

            for (let sfCategory of $scope.allCategories.categoryGroups) {
                for (let articleCategory of $scope.categories.records) {
                    if (sfCategory.name === articleCategory.DataCategoryGroupName) {

                        let rootCategory = {
                            expanded: false,
                            label: sfCategory.label,
                            name: sfCategory.name,
                            categories: [],
                            rootCategoryExpanded:
                            sfCategory
                        };
                        rootCategoryIndex.set(rootCategory.name,rootCategory);
                        rootCategories.push(rootCategory);
                        break;
                    }
                }
            }
            if (rootCategories.length > 0) {
                for (let articleCategory of $scope.categories.records) {
                    let categoryGroupName = articleCategory.DataCategoryGroupName;
                    let categoryName = articleCategory.DataCategoryName;
                    if (rootCategoryIndex.has(categoryGroupName)) {
                        let pathFragments = [];
                        let labelsInPathIndex = new Map();

                        let rootCategory = rootCategoryIndex.get(categoryGroupName);
                        buildPathFragments(rootCategory.rootCategoryExpanded, categoryName, pathFragments, labelsInPathIndex);

                        let label = labelsInPathIndex.has(categoryName) ? labelsInPathIndex.get(categoryName) : categoryName;
                        let category = {
                            label: label,
                            tooltip: pathFragments.reverse().join('>')
                        };
                        rootCategory.categories.push(category);
                    }
                }
                $scope.rootCategories = rootCategories;

                // timeout to call tooltip after leaf are rendered
                $timeout(function () {
                    $element.find(".leaf").tooltip({gravity: 'w'});
                });

            }
        };

        this.getSalesforceFeedback  = function(){
            knowledgeSalesforceFeedbackDetailsServiceInstance.confluenceFeedback()
                .catch( function(reason){
                    console.error(reason);
                    if(reason.data && reason.data.message){
                        $scope.error = reason.data.message;
                    }
                    else{
                        $scope.error = JSON.stringify(reason);
                    }
                } )
                .then(
                    function (result) {
                        if(!result){
                            return;
                        }

                        $scope.urlName = result.urlName;
                        $scope.pageVersion = result.pageVersion;
                        $scope.lastPublishedVersion = result.lastPublishedVersion;
                        $scope.publishedBy = result.publishedBy;
                        $scope.publishedByAvatar = result.publishedByAvatar;
                        $scope.publishedByFullName = result.publishedByFullName;
                        $scope.lastPublishedVersion = result.lastPublishedVersion;
                        $scope.formattedUpdatedDate = result.formattedUpdatedDate;

                        return knowledgeSalesforceFeedbackDetailsServiceInstance.salesforceFeedback();
                    }
                ).catch( function (reason) {
                console.error(reason);
                if(reason.data && reason.data.message){
                    $scope.error = reason.data.message;
                }
                else{
                    $scope.error = JSON.stringify(reason);
                }
            }).then( function(result) {
                if(!result){
                    return;
                }

                var stars = ['☆', '☆', '☆', '☆', '☆'];
                for(var i = 0; i < result.votes; i++){
                    stars[i] = '★';
                }
                $scope.stars = stars;
                $scope.views = result.views;
                $scope.changesInSalesforce = result.changesInSalesforce;

                $scope.knowledgeArticleVersionId = result.knowledgeArticleVersionId;
                $scope.draftKnowledgeArticleVersionId = result.draftKnowledgeArticleVersionId;
                $scope.onlineKnowledgeArticleVersionId = result.onlineKnowledgeArticleVersionId;
                $scope.knowledgeArticleVersionUrl = result.knowledgeArticleVersionUrl;
                $scope.onlineKnowledgeArticleVersionUrl = result.onlineKnowledgeArticleVersionUrl;
                $scope.draftKnowledgeArticleVersionUrl = result.draftKnowledgeArticleVersionUrl;

                knowledgeSalesforceFeedbackCommon.calculateKAVsToShow(
                    $scope,
                    $scope.knowledgeArticleVersionId,
                    $scope.draftKnowledgeArticleVersionId,
                    $scope.onlineKnowledgeArticleVersionId
                );
                knowledgeSalesforceFeedbackCommon.updateScopeWithArticleMetadata($scope, result);

                $scope.categories = result.categories;
                $scope.allCategories = result.allCategories;

                handleCategories();
            });
        };

        this.$onInit = function () {
            this.getSalesforceFeedback();
        };
    };

var knowledgeSalesforceFeedbackDetailsService = function($rootScope, $q, $http){
    return {
        cancel:
            knowledgeSalesforceFeedbackCommon.cancel.bind(knowledgeSalesforceFeedbackCommon ),
        salesforceFeedback:
            knowledgeSalesforceFeedbackCommon.salesforceFeedback.bind( knowledgeSalesforceFeedbackCommon, $q, $http),
        confluenceFeedback:
            knowledgeSalesforceFeedbackCommon.confluenceFeedback.bind( knowledgeSalesforceFeedbackCommon, $q, $http),
        editMeta:
            knowledgeSalesforceFeedbackCommon.editMeta.bind( knowledgeSalesforceFeedbackCommon, $q, $http),
        isChangedInSalesforce:
            knowledgeSalesforceFeedbackCommon.isChangedInSalesforce.bind( knowledgeSalesforceFeedbackCommon, $q, $http),
        metadata:
            knowledgeSalesforceFeedbackCommon.metadata.bind(knowledgeSalesforceFeedbackCommon, $q, $http),
        categories:
            knowledgeSalesforceFeedbackCommon.categories.bind(knowledgeSalesforceFeedbackCommon, $q, $http)
    }
};

function loadKnowledgeSalesforceFeedbackDetailsComponent( app ) {

    console.log('loadKnowledgeSalesforceFeedbackDetailsComponent called');
    app.factory("knowledgeSalesforceFeedbackDetailsService", knowledgeSalesforceFeedbackDetailsService);
    knowledgeSalesforceFeedbackDetailsService.$inject = ["$rootScope", "$q", "$http"];

    app.component( "knowledgeSalesforceFeedbackDetails",
        {
            templateUrl: AJS.Data.get("context-path") +
                '/download/resources/com.zagile.confluence.kb.knowledge-bridge:salesforce-feedback-resources/knowledge-sf-feedback-details.html',
            controller: ['$rootScope', '$scope', '$element', '$q', '$timeout', 'knowledgeSalesforceFeedbackDetailsService', knowledgeSalesforceFeedbackDetailsController]
        }
    );
}